package main

import (
	// Standard
	"database/sql"
	"errors"
	"time"
)

type Session struct {
	UUID    string
	UserID  int
	Created time.Time
}

func NewSession() (session Session, err error) {
	var rows *sql.Rows
	if rows, err = db.Query(`
		INSERT INTO public.session(uuid)
		VALUES(gen_random_uuid())
		RETURNING uuid, created`,
	); err != nil {
		return
	}
	defer rows.Close()

	if rows.Next() {
		rows.Scan(&session.UUID, &session.Created)
	}

	return
}

func (session *Session) Retrieve() (err error) {
	var rows *sql.Rows
	if rows, err = db.Query(`
		SELECT
			uuid, user_id, created
		FROM public.session
		WHERE
			uuid = $1
		`,
		session.UUID,
	); err != nil {
		return
	}
	defer rows.Close()

	if rows.Next() {
		rows.Scan(&session.UUID, &session.UserID, &session.Created)
	}

	return
}

func (session *Session) Authenticate(username, password string) (err error) {
	authenticated := false

	var rows *sql.Rows
	if rows, err = db.Query(`
		SELECT id
		FROM public.user
		WHERE
			username=$1 AND
			password=$2
		`,
		username,
		password,
	); err != nil {
		return
	}
	defer rows.Close()

	if rows.Next() {
		rows.Scan(&session.UserID)
		authenticated = session.UserID > 0
	}

	if authenticated {
		_, err = db.Exec("UPDATE public.session SET user_id=$1 WHERE uuid=$2", session.UserID, session.UUID)
		if err != nil {
			return
		}
	} else {
		err = errors.New("Invalid authentication")
	}

	return 
}
